@@ -16,8 +16,6 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id$ */
-
 #include "php.h"
 #include <errno.h>
 #include "ext/standard/flock_compat.h"
@@ -33,10 +31,6 @@
 #include "config.w32.h"
 #endif
 
-#ifdef NETWARE
-#include <netinet/in.h>
-#endif
-
 #ifndef HAVE_FLOCK
 PHPAPI int flock(int fd, int operation)
 {
